//##protect##"FileHeader"
/** @file InputEventLogic.c
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 05.12.2012
 *  @date    $LastChangedDate: 2013-02-15 13:44:41 +0100 (Fr, 15. Feb 2013) $
 *  @version $LastChangedRevision: 3857 $
 *
 *  @par DESCRIPTION
 *  This sample demonstrates how to use the interrupts on the digital inputs 
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

#include "StdAfx.h"

//##protect##"InputEventLogic_Head"
struct eventInterrupt_t
{
	DWORD dw_Overflow;
	DWORD dw_DigitalInputMask;
	DWORD dw_DigitalInputStatus;
	DWORD dw_InterruptCount;
};

static struct eventInterrupt_t interruptInfo;

/* Function called when an interrupt is generated */
void v_MyFunction(HANDLE h_DeviceHandle,
                  BYTE   b_FIFOOverflow,
                  DWORD  dw_InterruptSource,
                  DWORD  dw_InterruptMask,
                  PDWORD pdw_Args,
                  DWORD  dw_ArgsCount)
{
	UNREFERENCED_PARAMETER(h_DeviceHandle);
	UNREFERENCED_PARAMETER(dw_ArgsCount);

	/* If digital input event interrupt */
	if (dw_InterruptSource == 1)
	{
		interruptInfo.dw_Overflow = b_FIFOOverflow;
		interruptInfo.dw_DigitalInputMask = dw_InterruptMask;
		interruptInfo.dw_DigitalInputStatus = pdw_Args[0];                 // Digital input status at interrupt time
		interruptInfo.dw_InterruptCount += 1;
	}
}
//##protect##"InputEventLogic_Head"

int _tmain(int argc, _TCHAR *argv[])
{
	INT    i_ReturnValue = 0;
	HANDLE h_BoardHandle = NULL;
//##protect##"InputEventLogic_VariablesDeclaration"
	BYTE   b_LogicInterrupt = 0;
	DWORD  dw_DigitalInputMode1 = 0;
	DWORD  dw_DigitalInputMode2 = 0;
//##protect##"InputEventLogic_VariablesDeclaration"

	UNREFERENCED_PARAMETER(argc);
	UNREFERENCED_PARAMETER(argv);

	printf("/****************************************************************************\\\n"
	       "|*                          InputEventLogic sample                          *|\n"
	       "|****************************************************************************|\n"
	       "|* This sample demonstrates how to use the interrupts on the digital        *|\n"
	       "|* inputs                                                                   *|\n"
	       "\\****************************************************************************/\n\n");

//##protect##"InputEventLogic_Start"
	memset(&interruptInfo, 0, sizeof(struct eventInterrupt_t));
//##protect##"InputEventLogic_Start"

	i_ReturnValue = i_Utils_OpenBoard(&h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "i_Utils_OpenBoard failed: %d\n", i_ReturnValue);
		_getch();
		return i_ReturnValue;
	}

//##protect##"InputEventLogic_Body"
	b_LogicInterrupt = (BYTE)ll_Utils_ReadInteger(0, 1, "Which interrupt logic do you want to use (0 for OR, 1 for AND)?");

	printf("\n****   Each bit from 0 to 15 is representing an input channel.   ****\n");
	printf("\n****   You will have to set two mask to define how signals have  ****\n"); 
	printf("\n****   to be on each channel to generate an event interrupt.    ****\n\n");
	printf("\n +------------+-------------+-------------+-------------+-------------+"); 
	printf("\n |   Mode-1   |      0      |      1      |      0      |      1      |");
	printf("\n +------------+-------------+-------------+-------------+-------------+");
	printf("\n |   Mode-2   |      0      |      0      |      1      |      1      |");
	printf("\n +------------+-------------+-------------+-------------+-------------+");
	printf("\n | Interrupt  |   Disable   |  On rising  |  On falling | On rising   |");
	printf("\n |            |             |    edge     |   edge      | and falling |");
	printf("\n |            |             |             |             |    edges    |");
	printf("\n +------------+-------------+-------------+-------------+-------------+\n\n");

	do
	{
		printf("Mode-1, in hexadecimal, between 1 and 0xFFFF (without 0x) or equal to 0 if not used:");
		scanf_s("%x", &dw_DigitalInputMode1);
	} while ((dw_DigitalInputMode1 == 0) && (dw_DigitalInputMode1 > 0xFFFF));

	do
	{
		printf("Mode-2, in hexadecimal, between 1 and 0xFFFF (without 0x) or equal to 0 if not used:");
		scanf_s("%x", &dw_DigitalInputMode2);
	} while ((dw_DigitalInputMode2 == 0) && (dw_DigitalInputMode2 > 0xFFFF));

	i_ReturnValue = i_PCIe1032_SetBoardIntRoutine(h_BoardHandle, v_MyFunction);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s\n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}

	i_ReturnValue = i_PCIe1032_InitDigitalInputEventLogic(h_BoardHandle,
		                                                  b_LogicInterrupt,
		                                                  dw_DigitalInputMode1,
														  dw_DigitalInputMode2);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s\n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}

	i_ReturnValue = i_PCIe1032_StartDigitalInputEventLogic(h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s\n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}

	do
	{
		printf("\rInt #%lu, Overflow: %s, Event mask: %-8x, Input status: %-8x", interruptInfo.dw_InterruptCount,
                                                                                 (interruptInfo.dw_Overflow == 0 ? "no" : "yes"),
                                                                                 interruptInfo.dw_DigitalInputMask,
                                                                                 interruptInfo.dw_DigitalInputStatus);
		Sleep(50);
	} while (!_kbhit());
	// Catch the previous _kbhit
	_getch();

	i_ReturnValue = i_PCIe1032_StopDigitalInputEventLogic(h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s\n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}

	i_ReturnValue = i_PCIe1032_ReleaseDigitalInputEventLogic(h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s\n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}

	i_ReturnValue = i_PCIe1032_ResetBoardIntRoutine(h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s\n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}
//##protect##"InputEventLogic_Body"

	i_ReturnValue = i_PCIe1032_CloseBoard(h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "i_PCIe1032_CloseBoard failed: %d\n", i_ReturnValue);
		_getch();
		return i_ReturnValue;
	}

//##protect##"InputEventLogic_End"
//##protect##"InputEventLogic_End"

	printf("\nPress any key to exit the sample...\n");
	_getch();
	return i_ReturnValue;
}
