//##protect##"FileHeader"
/** @file Timer.c
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 05.12.2012
 *  @date    $LastChangedDate: 2012-12-07 10:27:39 +0100 (Fr, 07 Dez 2012) $
 *  @version $LastChangedRevision: 3636 $
 *
 *  @par DESCRIPTION
 *  This sample demonstrates how to configure, start and get the value of a timer 
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

#include "StdAfx.h"

//##protect##"Timer_Head"
//##protect##"Timer_Head"

int _tmain(int argc, _TCHAR *argv[])
{
	INT    i_ReturnValue = 0;
	HANDLE h_BoardHandle = NULL;
//##protect##"Timer_VariablesDeclaration"
	BYTE   b_EnableInterrupt = 0;
	BYTE   b_TimeBase = 0;
	WORD   w_ReloadValue = 0;
//##protect##"Timer_VariablesDeclaration"

	UNREFERENCED_PARAMETER(argc);
	UNREFERENCED_PARAMETER(argv);

	printf("/****************************************************************************\\\n"
	       "|*                               Timer sample                               *|\n"
	       "|****************************************************************************|\n"
	       "|* This sample demonstrates how to configure, start and get the value of    *|\n"
	       "|* a timer                                                                  *|\n"
	       "\\****************************************************************************/\n\n");

//##protect##"Timer_Start"
//##protect##"Timer_Start"

	i_ReturnValue = i_Utils_OpenBoard(&h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "i_Utils_OpenBoard failed: %d\n", i_ReturnValue);
		_getch();
		return i_ReturnValue;
	}

//##protect##"Timer_Body"
	/* Read the settings from the user input */
	b_EnableInterrupt = (BYTE)ll_Utils_ReadInteger(0, 1, "Do you want to enable interrupts (0: No, 1: Yes)?");
	b_TimeBase = (BYTE)ll_Utils_ReadInteger(0, 2, "Which time base do you want to use (0: Microseconds, 1: Milliseconds, 2: Seconds)?");
	w_ReloadValue = (WORD)ll_Utils_ReadInteger(0, 4095, "Which reload value do you want to use (0 to 4095)?");

	/* Enable interrupt routine if requested */
	if (b_EnableInterrupt != 0)
	{
		i_ReturnValue = i_PCIe1032_SetBoardIntRoutine(h_BoardHandle, v_Utils_InterruptRoutine);
		if (i_ReturnValue != 0)
		{
			fprintf(stderr, "%s \n", pc_PCIe1032_GetLastError(h_BoardHandle));
			i_PCIe1032_CloseBoard(h_BoardHandle);
			_getch();
			return i_ReturnValue;
		}
	}

	/* Start the timer */
	i_ReturnValue = i_PCIe1032_InitAndStartTimer(h_BoardHandle, b_TimeBase, w_ReloadValue, b_EnableInterrupt);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "%s \n", pc_PCIe1032_GetLastError(h_BoardHandle));
		i_PCIe1032_CloseBoard(h_BoardHandle);
		_getch();
		return i_ReturnValue;
	}
	else
	{
		printf("i_PCIe1032_InitAndStartTimer OK. \n");
		printf("Press the space key to trigger the timer. \n");
		printf("\n");
	}

	/* Loop until stopped */
	for (;;)
	{
		BYTE b_TimerStatus = 0;
		BYTE b_SoftwareTriggerStatus = 0;
		WORD w_TimerValue = 0;

		/* Get the timer status */
		i_ReturnValue = i_PCIe1032_ReadTimerStatus(h_BoardHandle, &b_TimerStatus, &b_SoftwareTriggerStatus, &w_TimerValue);
		if (i_ReturnValue != 0)
		{
			fprintf(stderr, "%s \n", pc_PCIe1032_GetLastError(h_BoardHandle));
			break;
		}
		printf("\rTimer value: %u", w_TimerValue);

		/* Process user events */
		if (_kbhit())
		{
			/* space bar */
			if (_getch() == ' ')
			{
				i_ReturnValue = i_PCIe1032_TriggerTimer(h_BoardHandle);
				if (i_ReturnValue != 0)
				{
					fprintf(stderr, "%s \n", pc_PCIe1032_GetLastError(h_BoardHandle));
					break;
				}
				else
					printf("\nThe timer has been triggered!\n\n");
			}
			else
				break;
		}

		Sleep(100);
	}

	/* Stop the timer */
	i_ReturnValue = i_PCIe1032_StopAndReleaseTimer(h_BoardHandle);
	if (i_ReturnValue != 0)
		fprintf(stderr, "%s \n", pc_PCIe1032_GetLastError(h_BoardHandle));
	else
		printf("i_PCIe1032_StopTimer OK \n");

	/* Reset the interrupt routine if needed */
	if (b_EnableInterrupt != 0)
	{
		i_ReturnValue = i_PCIe1032_ResetBoardIntRoutine(h_BoardHandle);
		if (i_ReturnValue != 0)
			fprintf(stderr, "%s \n", pc_PCIe1032_GetLastError(h_BoardHandle));
		else
			printf("i_PCIe1032_ResetBoardIntRoutine OK \n");
	}
//##protect##"Timer_Body"

	i_ReturnValue = i_PCIe1032_CloseBoard(h_BoardHandle);
	if (i_ReturnValue != 0)
	{
		fprintf(stderr, "i_PCIe1032_CloseBoard failed: %d\n", i_ReturnValue);
		_getch();
		return i_ReturnValue;
	}

//##protect##"Timer_End"
//##protect##"Timer_End"

	printf("\nPress any key to exit the sample...\n");
	_getch();
	return i_ReturnValue;
}
