//##protect##"FileHeader"
/** @file DllImport.cs
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 07.11.2012
 *  @date    $LastChangedDate: 2012-12-07 10:27:39 +0100 (Fr, 07. Dez 2012) $
 *  @version $LastChangedRevision: 3636 $
 *
 *  @par DESCRIPTION
 *  PCIe1032.dll imports
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2012 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe1032_NET
{
    /// <summary>
    /// Contains the prototypes of the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the user interruption function prototype
        /// </summary>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle, Byte overflow, UInt32 source, UInt32 mask, IntPtr args, UInt32 argsSize);

        #region "Generic"
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_GetNumberOfBoards(out Byte pb_NbrOfBoards);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_GetBoardInformation(Byte b_BoardIndex, UInt32 dw_IdentifierStringSize, out Byte[] pc_Identifier, out UInt32 pdw_UINumber, out UInt32 pdw_DeviceNumber, out UInt32 pdw_BusNumber, out UInt32 pdw_BaseAddress0, out UInt32 pdw_BaseAddress1, out UInt32 pdw_BaseAddress2, out UInt32 pdw_BaseAddress3, out Byte pb_Interrupt);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_OpenBoardViaIndex(Byte b_BoardIndex, out IntPtr ph_DeviceHandle);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_OpenBoardViaIdentifier(out Byte[] pc_Identifier, out IntPtr ph_DeviceHandle);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_CloseBoard(IntPtr h_DeviceHandle);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_GetDLLVersion(out Int32 pdw_Version);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_GetDeviceDriverVersion(IntPtr h_DeviceHandle, out Int32 pdw_Version);
        [DllImport("PCIe1032.dll")]
        public static extern IntPtr pc_PCIe1032_GetLastError(IntPtr h_DeviceHandle);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);
        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_ResetBoardIntRoutine(IntPtr h_DeviceHandle);
        #endregion

        #region "PCIe1032"

        #endregion

        #region "DigitalInput"

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_Read1DigitalInput(IntPtr deviceHandle, Byte channel, out Byte value);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_Read4DigitalInputs(IntPtr deviceHandle, Byte port, out Byte value);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_Read8DigitalInputs(IntPtr deviceHandle, Byte port, out Byte value);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_Read16DigitalInputs(IntPtr deviceHandle, Byte port, out UInt16 value);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_Read32DigitalInputs(IntPtr deviceHandle, out UInt32 value);

        #endregion
        #region "EventLogic"

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_InitDigitalInputEventLogic(IntPtr deviceHandle, Byte interruptLogic, UInt32 digitalInputMode1, UInt32 digitalInputMode2);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_StartDigitalInputEventLogic(IntPtr deviceHandle);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_StopDigitalInputEventLogic(IntPtr deviceHandle);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_ReleaseDigitalInputEventLogic(IntPtr deviceHandle);

        #endregion
        #region "Timer"

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_InitAndStartTimer(IntPtr deviceHandle, Byte timeBase, UInt16 reloadValue, Byte interruptFlag);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_ReadTimerValue(IntPtr deviceHandle, out UInt16 value);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_ReadTimerStatus(IntPtr deviceHandle, out Byte timerStatus, out Byte softwareTriggerStatus, out UInt16 timerValue);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_StopAndReleaseTimer(IntPtr deviceHandle);

        [DllImport("PCIe1032.dll")]
        public static extern short i_PCIe1032_TriggerTimer(IntPtr deviceHandle);
#endregion
        #region "Misc"
        //##protect##"DllImportMisc"
        //##protect##"DllImportMisc"
        #endregion
    }
}
